/*
 * 1-wire.c
 *
 *  Created on: 03.06.2011
 *      Author: kyznecov
 */

#include <avr/io.h>
#include <util/crc16.h>
#include <util/delay.h>
#include <avr/interrupt.h>

#define RX() bit_is_set(PINC,0)
#define TX1() PORTC &= ~_BV(1) //выходной сигнал инвертирован
#define TX0() PORTC |= _BV(1)

uint8_t frame[9] = {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};

uint8_t oneWireInit();
void oneWireTans(uint8_t);
void oneWireRecv(uint8_t);

uint16_t getT() {

	/*Чение адреса термо датчика*/
	if (oneWireInit()) return 255;
	_delay_us(110);			//Ожидание конца presence pulse
	oneWireTans(0x33);		//Чтение ROM подчиненного устройства
	oneWireRecv(8);			//Читаем ROM

	/*Расчет CRC*/
	uint8_t crcResult = 0;
	for (uint8_t i = 0; i<7;i++) {
		crcResult = _crc_ibutton_update(crcResult,frame[i]);
	}
	if (crcResult != frame[7]) return 254;

	/*даем команду конвертации температуры нужному устройсту*/
	if (oneWireInit()) return 253;
	_delay_us(110);			//Ожидание конца presence pulse
	oneWireTans(0x55);
	for (uint8_t i = 0; i<8; i++) {
		oneWireTans(frame[i]);
	}
	oneWireTans(0x44);		//Команда конвертации и записи температуры в регистр

	/*Чтение температуры*/
	if (oneWireInit()) return 252;
	_delay_us(110);			//Ожидание конца presence pulse
	oneWireTans(0x55);
	for (uint8_t i = 0; i<8; i++) {
		oneWireTans(frame[i]);
	}
	oneWireTans(0xBE);		//Комманда чтения регистра с температурой
	oneWireRecv(9);
	crcResult = 0;
	for (uint8_t i =0; i<8;i++) {
		crcResult = _crc_ibutton_update(crcResult,frame[i]);
	}
	if(crcResult != frame[8]) return 251;

	//Store integer
	uint16_t digit=frame[0]>>4;
	digit|=(frame[1]&0xf)<<4;
	//Store decimal digits
	uint8_t decimal=frame[0]&0xf;
	decimal= (decimal*5)/8;

	/*Возвращается только цело значение*/
	return ((uint16_t)digit);
}

/*Сброс и ожидание начала presence pulse*/
uint8_t oneWireInit() {
	cli();
	TX0();
	_delay_us(500);
	TX1();
	_delay_us(60);
	sei();
	return RX() ;//если на выводе 1 - ошибка
}

/* Передача 1 байта. Передача осуществляется
 * младшим битом вперед
 */
void oneWireTans(uint8_t data) {
	cli();
	for (uint8_t i = 0; i<8;i++) {
		TX0(); //импульс сброса (1-15мкс) - начало интервала передачи
		_delay_us(5);
		if (data & 0x01) {
			TX1();
		}
		else {
			TX0();
		}
		_delay_us(60);//пауза для считывания данных таблеткой (60-120мкс)
		TX1();//импульс сброса "1"(1мкс)
		_delay_us(10);
		data >>= 1;
	}
	sei();
}

/*Чтение length байт данных младшим байтом вперед*/
void oneWireRecv(uint8_t length) {
	cli();
	_delay_us(60);
	for (uint8_t i = 0;i<length;i++) {
		frame[i] = 0x00; //Очистка буфера приема
		/*Цикл для приема очередного байта*/
		for (uint8_t j = 0;j<8;j++) {
			frame[i] >>= 1;
			TX0();			//импульс сброса "0"(1-15мкс) - начало интервала приема
			_delay_us(10);
			TX1();			//импульс сброса "1"
			_delay_us(9);
			frame[i] |= RX() ? 0x80:0;
			_delay_us(50);
		}

	}
	sei();
}
